#
# HREDLEAK1 -- Fit and remove 'red leak' continuum from spectra in one file
#
# Modified 2007-05-08 Bill Wyatt 
#   Added peaklambda as defining peak of redleak spectrum and 
# peakorder as order of fit to use when the end of the spectrum
# approaches this value.
#
# By Bill Wyatt; modified by Doug Mink
# February 26, 2007
#
# Requires 'hectospec' package for gettime task
# Requires 'images.imfit' package for fit1d task
#

procedure hredleak1 (image)

string	image = "" {prompt="input file"}
real	lambda = 8520.0
real    peaklambda = 9350.0
bool	dounused = no    # if 'yes' correct even unused bands
string	fitfunc = "spline3"
int	order = 1
int     peakorder = 4
real	low_reject = 3.0
real	high_reject = 2.0
int	niterate = 10
bool	deltemp = yes	{prompt="Delete temporary files (y or n)"}
bool	saveold = yes	{prompt="Save unprocessed files to Notran/ (y or n)"}
bool	verbose = no	{prompt="Log progress (y or n)"}
bool	debug = no	{prompt="Log each aperture (y or n)"}

begin

	bool   db,       vb
	file   image_in, image_out
	file   newfile
	file   fitfile
	string section,  section2
	string sect1,    sect2
        int    fitorder
	int    deltapix
	int    nap
	int    lroot
	int    pixbeg,   pixend
        int    pixpeak
	int    naxis
	int    naxis2
	string keyval
	int    crpix1
	real   cdelt1,   crval1
	real   lomean
	real   himean

        real   lambda_end
        real   lam_delta = 225.0
	bool   variance
	string imroot
	file   imvar, imvarout

	string history,  hdata

	db = debug
	vb = verbose
        fitorder = order

# Make sure filename ends in .ms.fits
	image_in = image
	smatch (image_in, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (image_in, 1, lroot)
	    image_out = imroot // ".red.ms.fits"
	    smatch (image_in, ".fits")
	    if (!smatch.found) {
		image_in = image_in // ".fits"
		}
	    }
	else {
	    smatch (image_in, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (image_in, 1, lroot)
		image_in = imroot // ".ms.fits"
		image_out = imroot // ".red.ms.fits"
		}
	    else {
		imroot = image_in
		image_out = image_in // ".red.ms.fits"
		image_in = image_in // ".ms.fits"
		}
	    }

	if (!access (image_in)) {
	    print ("HREDLEAK1: ", image_in," does not exist")
	    return
	    }

# Make sure redleak not done twice
	imgetp.default = "0"
	imgetp (image_in, "REDLEAK")
	if (imgetp.value != "0") {
	    printf("REDLEAK already done on %s\n", image_in)
	    return
	    }

	if (access (image_out)) {
	    delete (image_out)
	    }

	naxis2  = 1

# get key values from header
	imgetp.verbose = no
	imgetp.default = "0"
	imgetp (image_in, "i_naxis")
	if (imgetp.value == "0") {
	    printf("redleakcorr: file %s: no NAXIS?\n", image_in)
	    error(1,"exiting...")
	    }
	naxis = int (imgetp.value)

	imgetp.default = "0"
	imgetp (image_in, "i_naxis1")
	if (imgetp.value == "0") {
	    printf("redleakcorr: file %s: no NAXIS1?\n", image_in)
	    error(1,"exiting...")
	    }
	pixend = int (imgetp.value)

	imgetp.default = "0"
	imgetp (image_in, "i_naxis2")
	if (imgetp.value == "0") {
	    nap = 1
	    }
	if (strlen (imgetp.value) > 0) 
	    nap = int (imgetp.value)
	else
	    nap = 1

	imgetp.default = "0"
	imgetp (image_in, "CRPIX1")
	if (imgetp.value == "0") {
	    printf("HREDLEAK1: file %s: no CRPIX1\n", image_in)
	    error(1,"exiting...")
	    }
	crpix1 = int(imgetp.value)

	imgetp.default = "0"
	imgetp (image_in, "CDELT1")
	if (imgetp.value == "0") {
	    imgetp (image_in, "CD1_1")
	    if (imgetp.value == "0") {
		printf("HREDLEAK1: file %s: no CDELT1 or CD1_1\n", image_in)
		error(1,"exiting...")
		}
	    }
	cdelt1 = real(imgetp.value)

	imgetp.default = "0"
	imgetp (image_in, "CRVAL1")
	if (imgetp.value == "0") {
	    printf("HREDLEAK1: file %s: no CRVAL1\n", image_in)
	    error(1,"exiting...")
	    }
	crval1 = real(imgetp.value)

       lambda_end = (pixend - crpix1) * cdelt1 + crval1

	pixbeg  = int((lambda - crval1)/cdelt1 + crpix1 - 1)
	pixpeak = int((peaklambda - 100. - crval1)/cdelt1 + crpix1 - 1)

# most of output is a copy of the input
      imcopy (image_in, image_out, verbose=no)

# if grating tilt puts beginning of red leak too close to end, skip
    if(lambda_end < lambda + lam_delta) {
      if (vb)
         printf("HREDLEAK1: lambda end= %.1f, No correction above %.1f\n", 
            lambda_end, lambda
         hdata = "spectrum end at " // str(lambda_end) // ", No correction"
    }
   else {
	if (vb)
            printf("HREDLEAK1: lam= %.1f, sec [%d:%d], crpix= %d, crval= %.1f cdelt= %.3f\n",
		   lambda, pixbeg, pixend, crpix1, crval1, cdelt1)

#  Check for existence of variance spectrum
	imvar = imroot // "_v.ms.fits"
	if (access (imvar)) {
	    variance = yes
	    imcopy (imvar, imvarout, verbose=no)

# Check to see whether fiber color throughput correction has been done
	    imgetp.default = "0"
	    imgetp (imvar, "REDLEAK")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HREDLEAK1: ", imvar, " has had sky absorption removed")
		    print (" REDLEAK = ", imgetp.value)
		    }
		variance = no
		}

	    }
	else {
	    variance = no
	    }
	fitfile  = osfn("tmp$") // mktemp("drdlk_") // ".fits"
	delete (fitfile,  ver-, >& "dev$null")

    # if spectrum end is near or beyond the red leak peak, use higher order
        if (lambda_end > peaklambda - 100.) {
            fitorder = peakorder
        }

	hdata   = "lambda " // str(int(lambda)) // "-" // str(int(lambda_end)) 
        hdata   = hdata // " fit " // fitfunc // " O(" // str(fitorder) // ")"

# Loop here for multiple apertures
	for (i = 1; i <= nap; i = i+1) {
	
	    imgetp.default = "0"
	    imgetp (image_in, "APID" // str(i))
	    keyval = imgetp.value
	    if (! dounused && strlen(keyval) > 5 && substr(keyval,1,6) == "unused") {
		if (vb) {
	  	    printf("HRDLEAK1: Aperture %d not used, not fit\n", i, " ")
	  	    next
	  	    }
	  	}
	    section = "[" // str(pixbeg) // ":" // str(pixend) // "," // str(i) // "]"
            
	    section2= "[" // str(pixbeg) // ":" // str(min(pixbeg+50,pixend)) // "," // str(i) // "]"

	    fit1d (image_in // section,  fitfile,  "fit",  axis=1,
	      inter-,                sample="*",              naver=1,
	      function=fitfunc,      order=fitorder,          grow=0.,
	      low_reject=low_reject, high_reject=high_reject, niterate=niterate)
	
# If the continuum trends downwards, there is no (significant) red leak and
# the subtraction should not be performed
#
# WFW 2006-09-11 -  unless the grating tilt is redwards enough that the
# redleak turnover (at ~ 9350A) is within the spectrum.

            if ( pixpeak >= pixend ) {
	    	deltapix = (pixend - pixbeg) / 10
		if(deltapix < 3) {
		    deltapix = min(3,pixend-pixbeg+1)
		}

           # get mean of first 10% of fit
  	        sect1 = "["//str(pixbeg)//":"//str(pixbeg+deltapix-1) // "," //str(i)// "]"
	        imstatistics(fitfile // sect1, 
	           format=no, fields="mean", nclip=0, lower=INDEF, upper=INDEF) |
	        scanf("%f", lomean)
	
           # get mean of last 10% of fit
	        sect2 = "["//str(pixend-deltapix+1)//":"//str(pixend)//","//str(i)// "]"
	        imstatistics(fitfile // sect2,
	           format=no, fields="mean", nclip=0, lower=INDEF, upper=INDEF) |
	        scanf("%f", himean)

	        if (vb) {
		    printf("rdlc(%3d): Fit mean%-15s= %6.1f  mean%-15s= %6.1f", 
	                i, sect1, lomean, sect2, himean)
	  	}

            }
            else {  # force subtraction of fit 
               himean = 2.
               lomean = 1.
            }
	    if (himean > lomean) {

		gettime(usesec=yes,verbose=no)

	  # subtract off the fit
		imexpr("a - b + lov(c)",  image_out // section, 
		       image_in // section,  fitfile // section,  
                       fitfile // section2, verbose-)
		if (vb) printf(" corr")
	    }
	    if (vb) printf("\n")

	  # Update variance for this aperture
	    if (variance) {
		imexpr ("a + ((stdev (b - c)) ** 2)",imvarout // section,
			imvar // section, image_in // section,
			fitfile // section, verbose-)
	    }

# End aperture loop
	}
 
      delete (fitfile,  ver-, >& "dev$null")

    }

# Move original file to Nored directory
    if (saveold) {
        if (!access ("Nored/")) {
	    mkdir ("Nored")
	}
	newfile = "Nored/" // image_in
	if (access (newfile)) {
	    imdelete (newfile)
	}
	movefiles (image_in, "Nored")
	if (variance)
	    movefiles (imvar, "Nored")
    }
    else {
	delete (image_in)
    }

# Rename result file to original file name
    imrename (image_out, image_in)

    history = gettime.date// " " // gettime.time // " " // hdata
    hedit (image_in, "REDLEAK", history, add+, update+, show-, verify-)
    if (variance) {
	imrename (imvarout, imvar)
	hedit (imvar, "REDLEAK", history, add+, update+, show-, verify-)
	}
	
end

# Jun  9 2005	New task

# Feb 26 2007	Add code to compute variance spectrum
